@@ -1,5 +1,6 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+"""International Association for the Properties of Water and Steam (IAPWS)."""
 
 import os
 
@@ -11,7 +12,7 @@ from .iapws08 import SeaWater  # noqa
 from .humidAir import HumidAir  # noqa
 from .ammonia import H2ONH3  # noqa
 
-basepath = os.path.dirname(__file__)
+basepath = os.path.abspath(os.path.dirname(__file__))
 with open(os.path.join(basepath, 'VERSION')) as version_file:
     __version__ = version_file.read().strip()
 
@@ -241,4 +242,4 @@ __doi__ = {
                   "Seawater",
          "ref": "2016",
          "doi": ""},
-        }
+}
