@@ -1,5 +1,9 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+# pylint: disable=invalid-name
+# pylint: disable=too-many-lines, too-many-locals, too-many-statements
+# pylint: disable=too-many-branches, too-many-boolean-expressions
+
 """
 Miscelaneous IAPWS standards. This module include:
 
@@ -18,10 +22,10 @@ Miscelaneous IAPWS standards. This module include:
     * :func:`_D2O_Viscosity`: Viscosity correlation for heavy water
     * :func:`_D2O_ThCond`: Thermal conductivity correlation for heavy water
     * :func:`_D2O_Tension`: Surface tension correlation for heavy water
-    * :func:`_D2O_Sublimation_Pressure`: Sublimation Pressure correlation for
-    heavy water
+    * :func:`_D2O_Sublimation_Pressure`: Sublimation Pressure correlation
+      for heavy water
     * :func:`_D2O_Melting_Pressure`: Melting Pressure correlation for heavy
-    water
+      water
     * :func:`_Henry`: Henry constant for liquid-gas equilibrium
     * :func:`_Kvalue`: Vapor-liquid distribution constant
 """
@@ -29,10 +33,10 @@ Miscelaneous IAPWS standards. This module include:
 from __future__ import division
 
 from cmath import log as log_c
-from math import log, exp, tan, atan, acos, sin, pi, log10, copysign
+from math import log, exp, tan, atan, acos, sin, pi, log10
 import warnings
 
-from scipy.optimize import minimize
+from scipy.optimize import newton
 
 
 # Constants
@@ -45,7 +49,7 @@ Tc = 647.096      # K
 Pc = 22.064       # MPa
 rhoc = 322.       # kg/m³
 Tc_D2O = 643.847  # K
-Pc_D2O = 21.671   # MPa
+Pc_D2O = 21.6618   # MPa
 rhoc_D2O = 355.9999698294    # kg/m³
 
 Tt = 273.16       # K
@@ -92,7 +96,7 @@ def _Ice(T, P):
             * gtp: [∂²g/∂T∂P]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * T ≤ 273.16
@@ -130,7 +134,7 @@ def _Ice(T, P):
         if Psub > P:
             # Zone Gas
             warnings.warn("Metastable ice in vapor region")
-    elif 251.165 < T:
+    elif T > 251.165:
         Pmel = _Melting_Pressure(T)
         if Pmel < P:
             # Zone Liquid
@@ -241,12 +245,12 @@ def _Liquid(T, P=0.1):
             * alfav: Cubic expansion coefficient, [1/K]
             * xkappa : Isothermal compressibility, [1/MPa]
             * ks: Isentropic compressibility, [1/MPa]
-            * mu: Viscosity, [mPas]
+            * mu: Viscosity, [Pas]
             * k: Thermal conductivity, [W/mK]
             * epsilon: Dielectric constant, [-]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 253.15 ≤ T ≤ 383.15
@@ -266,11 +270,11 @@ def _Liquid(T, P=0.1):
     # Check input in range of validity
     if T <= 253.15 or T >= 383.15 or P < 0.1 or P > 0.3:
         raise NotImplementedError("Incoming out of bound")
-    elif P != 0.1:
+    if P != 0.1:
         # Raise a warning if the P value is extrapolated
         warnings.warn("Using extrapolated values")
 
-    R = 0.46151805   # kJ/kgK
+    Rg = 0.46151805   # kJ/kgK
     Po = 0.1
     Tr = 10
     tau = T/Tr
@@ -290,39 +294,39 @@ def _Liquid(T, P=0.1):
     n = [None, 4, 5, 7, None, None, 4, 5, 7, 8, 9, 1, 3, 5, 6, 7]
     m = [None, 2, 3, 4, 5, 1, 2, 3, 4, 5, 6, 1, 3, 4, 5, 6, 7, 9]
 
-    suma1 = sum([a[i]*alfa**n[i] for i in range(1, 4)])
-    suma2 = sum([b[i]*beta**m[i] for i in range(1, 5)])
-    go = R*Tr*(c[1]+c[2]*tau+c[3]*tau*log(tau)+suma1+suma2)
+    suma1 = sum(a[i]*alfa**n[i] for i in range(1, 4))
+    suma2 = sum(b[i]*beta**m[i] for i in range(1, 5))
+    go = Rg*Tr*(c[1]+c[2]*tau+c[3]*tau*log(tau)+suma1+suma2)
 
-    suma1 = sum([a[i]*alfa**n[i] for i in range(6, 11)])
-    suma2 = sum([b[i]*beta**m[i] for i in range(5, 11)])
-    vo = R*Tr/Po/1000*(a[5]+suma1+suma2)
+    suma1 = sum(a[i]*alfa**n[i] for i in range(6, 11))
+    suma2 = sum(b[i]*beta**m[i] for i in range(5, 11))
+    vo = Rg*Tr/Po/1000*(a[5]+suma1+suma2)
 
-    suma1 = sum([a[i]*alfa**n[i] for i in range(11, 16)])
-    suma2 = sum([b[i]*beta**m[i] for i in range(11, 18)])
-    vpo = R*Tr/Po**2/1000*(suma1+suma2)
+    suma1 = sum(a[i]*alfa**n[i] for i in range(11, 16))
+    suma2 = sum(b[i]*beta**m[i] for i in range(11, 18))
+    vpo = Rg*Tr/Po**2/1000*(suma1+suma2)
 
-    suma1 = sum([n[i]*a[i]*alfa**(n[i]+1) for i in range(1, 4)])
-    suma2 = sum([m[i]*b[i]*beta**(m[i]+1) for i in range(1, 5)])
-    so = -R*(c[2]+c[3]*(1+log(tau))+suma1-suma2)
+    suma1 = sum(n[i]*a[i]*alfa**(n[i]+1) for i in range(1, 4))
+    suma2 = sum(m[i]*b[i]*beta**(m[i]+1) for i in range(1, 5))
+    so = -Rg*(c[2]+c[3]*(1+log(tau))+suma1-suma2)
 
-    suma1 = sum([n[i]*(n[i]+1)*a[i]*alfa**(n[i]+2) for i in range(1, 4)])
-    suma2 = sum([m[i]*(m[i]+1)*b[i]*beta**(m[i]+2) for i in range(1, 5)])
-    cpo = -R*(c[3]+tau*suma1+tau*suma2)
+    suma1 = sum(n[i]*(n[i]+1)*a[i]*alfa**(n[i]+2) for i in range(1, 4))
+    suma2 = sum(m[i]*(m[i]+1)*b[i]*beta**(m[i]+2) for i in range(1, 5))
+    cpo = -Rg*(c[3]+tau*suma1+tau*suma2)
 
-    suma1 = sum([n[i]*a[i]*alfa**(n[i]+1) for i in range(6, 11)])
-    suma2 = sum([m[i]*b[i]*beta**(m[i]+1) for i in range(5, 11)])
-    vto = R/Po/1000*(suma1-suma2)
+    suma1 = sum(n[i]*a[i]*alfa**(n[i]+1) for i in range(6, 11))
+    suma2 = sum(m[i]*b[i]*beta**(m[i]+1) for i in range(5, 11))
+    vto = Rg/Po/1000*(suma1-suma2)
 
     # This properties are only neccessary for computing thermodynamic
     # properties at pressures different from 0.1 MPa
-    suma1 = sum([n[i]*(n[i]+1)*a[i]*alfa**(n[i]+2) for i in range(6, 11)])
-    suma2 = sum([m[i]*(m[i]+1)*b[i]*beta**(m[i]+2) for i in range(5, 11)])
-    vtto = R/Tr/Po/1000*(suma1+suma2)
+    suma1 = sum(n[i]*(n[i]+1)*a[i]*alfa**(n[i]+2) for i in range(6, 11))
+    suma2 = sum(m[i]*(m[i]+1)*b[i]*beta**(m[i]+2) for i in range(5, 11))
+    vtto = Rg/Tr/Po/1000*(suma1+suma2)
 
-    suma1 = sum([n[i]*a[i]*alfa**(n[i]+1) for i in range(11, 16)])
-    suma2 = sum([m[i]*b[i]*beta**(m[i]+1) for i in range(11, 18)])
-    vpto = R/Po**2/1000*(suma1-suma2)
+    suma1 = sum(n[i]*a[i]*alfa**(n[i]+1) for i in range(11, 16))
+    suma2 = sum(m[i]*b[i]*beta**(m[i]+1) for i in range(11, 18))
+    vpto = Rg/Po**2/1000*(suma1-suma2)
 
     if P != 0.1:
         go += vo*(P-0.1)
@@ -330,7 +334,7 @@ def _Liquid(T, P=0.1):
         cpo -= T*vtto*(P-0.1)
         vo -= vpo*(P-0.1)
         vto += vpto*(P-0.1)
-        vppo = 3.24e-10*R*Tr/0.1**3
+        vppo = 3.24e-10*Rg*Tr/0.1**3
         vpo += vppo*(P-0.1)
 
     h = go+T*so
@@ -367,47 +371,24 @@ def _Liquid(T, P=0.1):
     a = [None, 280.68, 511.45, 61.131, 0.45903]
     b = [None, -1.9, -7.7, -19.6, -40]
     T_ = T/300
-    mu = sum([a[i]*T_**b[i] for i in range(1, 5)])/1e6
+    mu = sum(a[i]*T_**b[i] for i in range(1, 5))/1e6
     propiedades["mu"] = mu
 
     # Thermal conductivity correlation, Eq 8
     c = [None, 1.6630, -1.7781, 1.1567, -0.432115]
     d = [None, -1.15, -3.4, -6.0, -7.6]
-    k = sum([c[i]*T_**d[i] for i in range(1, 5)])
+    k = sum(c[i]*T_**d[i] for i in range(1, 5))
     propiedades["k"] = k
 
     # Dielectric constant correlation, Eq 9
     e = [None, -43.7527, 299.504, -399.364, 221.327]
     f = [None, -0.05, -1.47, -2.11, -2.31]
-    epsilon = sum([e[i]*T_**f[i] for i in range(1, 5)])
+    epsilon = sum(e[i]*T_**f[i] for i in range(1, 5))
     propiedades["epsilon"] = epsilon
 
     return propiedades
 
 
-class _Supercooled_minimize(object):
-
-    def __init__(self, L, omega, xmin, xmax):
-        self.L = L
-        self.omega = omega
-        self.xmin = xmin
-        self.xmax = xmax
-        self.f_inner_value_sign = 0.0
-
-    def f(self, x):
-        f_inner_value = self.L+log(x/(1-x))+self.omega*(1-2*x)
-        self.f_inner_value_sign = copysign(1.0, f_inner_value)
-        return abs(f_inner_value)
-
-    def jac(self, x):
-        return self.f_inner_value_sign*(1/(x*(1 - x)) - 2*self.omega)
-
-    @property
-    def x(self):
-        return minimize(self.f, x0=((self.xmin+self.xmax)/2,),
-                        bounds=((self.xmin, self.xmax),), jac=self.jac)["x"][0]
-
-
 # IAPWS-15 for supercooled liquid water
 def _Supercooled(T, P):
     """Guideline on thermodynamic properties of supercooled water
@@ -439,7 +420,7 @@ def _Supercooled(T, P):
             * w: Speed of sound, [m/s²]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * Tm ≤ T ≤ 300
@@ -448,18 +429,19 @@ def _Supercooled(T, P):
     The minimum temperature in range of validity is the melting temperature, it
     depend of pressure
 
+    Raise :class:`RuntimeError` if solution isn't founded
+
     Examples
     --------
-    >>> liq = _Supercooled(235.15, 0.101325)
+    >>> liq = _supercooled(235.15, 0.101325)
     >>> liq["rho"], liq["cp"], liq["w"]
     968.09999 5.997563 1134.5855
 
     References
     ----------
-    IAPWS, Guideline on Thermodynamic Properties of Supercooled Water,
+    iapws, guideline on thermodynamic properties of supercooled water,
     http://iapws.org/relguide/Supercooled.html
     """
-
     # Check input in range of validity
     if P < 198.9:
         Tita = T/235.15
@@ -474,8 +456,8 @@ def _Supercooled(T, P):
     # Parameters, Table 1
     Tll = 228.2
     rho0 = 1081.6482
-    R = 0.461523087
-    pi0 = 300e3/rho0/R/Tll
+    Rg = 0.461523087
+    pi0 = 300e3/rho0/Rg/Tll
     omega0 = 0.5212269
     L0 = 0.76317954
     k0 = 0.072158686
@@ -484,7 +466,7 @@ def _Supercooled(T, P):
 
     # Reducing parameters, Eq 2
     tau = T/Tll-1
-    p = P*1000/rho0/R/Tll
+    p = P*1000/rho0/Rg/Tll
     tau_ = tau+1
     p_ = p+pi0
 
@@ -533,7 +515,28 @@ def _Supercooled(T, P):
         xmin = 0.99*exp(-50/49*L-omega)
         xmax = min(1.1*exp(-L-omega), 0.0101)
 
-    x = _Supercooled_minimize(L=L, omega=omega, xmin=xmin, xmax=xmax).x
+    # Eq 8
+    def f(x):
+        "Function for iterative calculation"
+        if x < xmin:
+            x = xmin
+        if x > xmax:
+            x = xmax
+        return L+log(x/(1-x))+omega*(1-2*x)
+
+    x = None
+    for xo in (xmin, xmax, (xmin+xmax)/2):
+        try:
+            x, sol = newton(f, xo, full_output=True)
+        except RuntimeError:
+            pass
+        else:
+            if sol.converged:
+                break
+
+    # Exit when solution don't found
+    if not x:
+        raise RuntimeError("Solution don't found")
 
     # Eq 12
     fi = 2*x-1
@@ -557,8 +560,8 @@ def _Supercooled(T, P):
     prop["g"] = phir+(tau+1)*(x*L+x*log(x)+(1-x)*log(1-x)+omega*x*(1-x))
 
     # Eq 14
-    prop["s"] = -R*((tau+1)/2*Lt*(fi+1) +
-                    (x*L+x*log(x)+(1-x)*log(1-x)+omega*x*(1-x))+phirt)
+    prop["s"] = -Rg*((tau+1)/2*Lt*(fi+1)
+                    + (x*L+x*log(x)+(1-x)*log(1-x)+omega*x*(1-x))+phirt)
 
     # Basic derived state properties
     prop["h"] = prop["g"]+T*prop["s"]
@@ -566,12 +569,12 @@ def _Supercooled(T, P):
     prop["a"] = prop["u"]-T*prop["s"]
 
     # Eq 15
-    prop["xkappa"] = prop["rho"]/rho0**2/R*1000/Tll*(
+    prop["xkappa"] = prop["rho"]/rho0**2/Rg*1000/Tll*(
         (tau+1)/2*(Xi*(Lp-omega0*fi)**2-(fi+1)*Lpp)-phirpp)
     prop["alfap"] = prop["rho"]/rho0/Tll*(
-        Ltp/2*(tau+1)*(fi+1) + (omega0*(1-fi**2)/2+Lp*(fi+1))/2 -
-        (tau+1)*Lt/2*Xi*(Lp-omega0*fi) + phirtp)
-    prop["cp"] = -R*(tau+1)*(Lt*(fi+1)+(tau+1)/2*(Ltt*(fi+1)-Lt**2*Xi)+phirtt)
+        Ltp/2*(tau+1)*(fi+1) + (omega0*(1-fi**2)/2+Lp*(fi+1))/2
+        - (tau+1)*Lt/2*Xi*(Lp-omega0*fi) + phirtp)
+    prop["cp"] = -Rg*(tau+1)*(Lt*(fi+1)+(tau+1)/2*(Ltt*(fi+1)-Lt**2*Xi)+phirtt)
 
     # Eq 16
     prop["cv"] = prop["cp"]-T*prop["alfap"]**2/prop["rho"]/prop["xkappa"]*1e3
@@ -595,7 +598,7 @@ def _Sublimation_Pressure(T):
         Pressure at sublimation line, [MPa]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 50 ≤ T ≤ 273.16
@@ -618,8 +621,8 @@ def _Sublimation_Pressure(T):
         for ai, expi in zip(a, expo):
             suma += ai*Tita**expi
         return exp(suma/Tita)*Pt
-    else:
-        raise NotImplementedError("Incoming out of bound")
+
+    raise NotImplementedError("Incoming out of bound")
 
 
 def _Melting_Pressure(T, ice="Ih"):
@@ -640,7 +643,7 @@ def _Melting_Pressure(T, ice="Ih"):
         Pressure at sublimation line, [MPa]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 251.165 ≤ T ≤ 715
@@ -691,8 +694,8 @@ def _Melting_Pressure(T, ice="Ih"):
         Tref = 355
         Pref = 2216.000
         Tita = T/Tref
-        P = Pref*exp(1.73683*(1-1./Tita)-0.544606e-1*(1-Tita**5) +
-                     0.806106e-7*(1-Tita**22))
+        P = Pref*exp(1.73683*(1-1./Tita)-0.544606e-1*(1-Tita**5)
+                     + 0.806106e-7*(1-Tita**22))
     else:
         raise NotImplementedError("Incoming out of bound")
     return P
@@ -735,19 +738,19 @@ def _Viscosity(rho, T, fase=None, drho=None):
 
     # Eq 11
     H = [1.67752, 2.20462, 0.6366564, -0.241605]
-    mu0 = 100*Tr**0.5/sum([Hi/Tr**i for i, Hi in enumerate(H)])
+    mu0 = 100*Tr**0.5/sum(Hi/Tr**i for i, Hi in enumerate(H))
 
     # Eq 12
-    I = [0, 1, 2, 3, 0, 1, 2, 3, 5, 0, 1, 2, 3, 4, 0, 1, 0, 3, 4, 3, 5]
-    J = [0, 0, 0, 0, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 3, 4, 4, 5, 6, 6]
+    li = [0, 1, 2, 3, 0, 1, 2, 3, 5, 0, 1, 2, 3, 4, 0, 1, 0, 3, 4, 3, 5]
+    lj = [0, 0, 0, 0, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 3, 4, 4, 5, 6, 6]
     Hij = [0.520094, 0.850895e-1, -0.108374e1, -0.289555, 0.222531, 0.999115,
            0.188797e1, 0.126613e1, 0.120573, -0.281378, -0.906851, -0.772479,
            -0.489837, -0.257040, 0.161913, 0.257399, -0.325372e-1, 0.698452e-1,
            0.872102e-2, -0.435673e-2, -0.593264e-3]
-    mu1 = exp(Dr*sum([(1/Tr-1)**i*h*(Dr-1)**j for i, j, h in zip(I, J, Hij)]))
+    mu1 = exp(Dr*sum((1/Tr-1)**i*h*(Dr-1)**j for i, j, h in zip(li, lj, Hij)))
 
     # Critical enhancement
-    if fase and drho:
+    if rho and fase and drho:
         qc = 1/1.9
         qd = 1/1.1
 
@@ -824,24 +827,24 @@ def _ThCond(rho, T, fase=None, drho=None):
 
     # Eq 16
     no = [2.443221e-3, 1.323095e-2, 6.770357e-3, -3.454586e-3, 4.096266e-4]
-    k0 = Tr**0.5/sum([n/Tr**i for i, n in enumerate(no)])
+    k0 = Tr**0.5/sum(n/Tr**i for i, n in enumerate(no))
 
     # Eq 17
-    I = [0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4,
-         4, 4, 4, 4, 4]
-    J = [0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 0,
-         1, 2, 3, 4, 5]
+    li = [0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4,
+          4, 4, 4, 4, 4]
+    lj = [0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 0,
+          1, 2, 3, 4, 5]
     nij = [1.60397357, -0.646013523, 0.111443906, 0.102997357, -0.0504123634,
            0.00609859258, 2.33771842, -2.78843778, 1.53616167, -0.463045512,
            0.0832827019, -0.00719201245, 2.19650529, -4.54580785, 3.55777244,
            -1.40944978, 0.275418278, -0.0205938816, -1.21051378, 1.60812989,
            -0.621178141, 0.0716373224, -2.7203370, 4.57586331, -3.18369245,
            1.1168348, -0.19268305, 0.012913842]
-    k1 = exp(d*sum([(1/Tr-1)**i*n*(d-1)**j for i, j, n in zip(I, J, nij)]))
+    k1 = exp(d*sum((1/Tr-1)**i*n*(d-1)**j for i, j, n in zip(li, lj, nij)))
 
     # Critical enhancement
-    if fase:
-        R = 0.46151805
+    if rho and fase:
+        Rg = 0.46151805
 
         if not drho:
             # Industrial formulation
@@ -861,7 +864,7 @@ def _ThCond(rho, T, fase=None, drho=None):
             else:
                 ai = [1.11999926419994, 0.595748562571649, 9.88952565078920,
                       -10.3255051147040, 4.66861294457414, -0.503243546373828]
-            drho = 1/sum([a*d**i for i, a in enumerate(ai)])*rhoc/Pc
+            drho = 1/sum(a*d**i for i, a in enumerate(ai))*rhoc/Pc
 
         DeltaX = d*(Pc/rhoc*fase.drhodP_T-Pc/rhoc*drho*1.5/Tr)
         if DeltaX < 0:
@@ -878,7 +881,7 @@ def _ThCond(rho, T, fase=None, drho=None):
                 1-exp(-1/(1/y+y**2/3/d**2))))
 
         # Eq 18
-        k2 = 177.8514*d*fase.cp/R*Tr/fase.mu*1e-6*Z
+        k2 = 177.8514*d*fase.cp/Rg*Tr/fase.mu*1e-6*Z
 
     else:
         # No critical enhancement
@@ -903,7 +906,7 @@ def _Tension(T):
         Surface tension, [N/m]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 248.15 ≤ T ≤ 647
@@ -927,8 +930,8 @@ def _Tension(T):
 
         # The equation give surface tension in mN/m², converted to N/m²
         return 1e-3*sigma
-    else:
-        raise NotImplementedError("Incoming out of bound")
+
+    raise NotImplementedError("Incoming out of bound")
 
 
 def _Dielectric(rho, T):
@@ -947,7 +950,7 @@ def _Dielectric(rho, T):
         Dielectric constant, [-]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 238 ≤ T ≤ 1200
@@ -977,23 +980,23 @@ def _Dielectric(rho, T):
 
     d = rho/rhoc
     Tr = Tc/T
-    I = [1, 1, 1, 2, 3, 3, 4, 5, 6, 7, 10, None]
-    J = [0.25, 1, 2.5, 1.5, 1.5, 2.5, 2, 2, 5, 0.5, 10, None]
-    n = [0.978224486826, -0.957771379375, 0.237511794148, 0.714692244396,
-         -0.298217036956, -0.108863472196, .949327488264e-1, -.980469816509e-2,
-         .165167634970e-4, .937359795772e-4, -.12317921872e-9,
-         .196096504426e-2]
+    li = [1, 1, 1, 2, 3, 3, 4, 5, 6, 7, 10]
+    lj = [0.25, 1, 2.5, 1.5, 1.5, 2.5, 2, 2, 5, 0.5, 10]
+    ni = [0.978224486826, -0.957771379375, 0.237511794148, 0.714692244396,
+          -0.298217036956, -0.108863472196, 0.949327488264e-1,
+          -.980469816509e-2, 0.165167634970e-4, 0.937359795772e-4,
+          -0.12317921872e-9, 0.196096504426e-2]
 
-    g = 1+n[11]*d/(Tc/228/Tr-1)**1.2
-    for i in range(11):
-        g += n[i]*d**I[i]*Tr**J[i]
+    g = 1+ni[11]*d/(Tc/228/Tr-1)**1.2
+    for n, i, j in zip(ni, li, lj):
+        g += n * d**i * Tr**j
     A = Na*mu**2*rho*g/M*1000/epsilon0/k/T
     B = Na*alfa*rho/3/M*1000/epsilon0
     e = (1+A+5*B+(9+2*A+18*B+A**2+10*A*B+9*B**2)**0.5)/4/(1-B)
     return e
 
 
-def _Refractive(rho, T, l=0.5893):
+def _Refractive(rho, T, lr=0.5893):
     """Equation for the refractive index
 
     Parameters
@@ -1002,7 +1005,7 @@ def _Refractive(rho, T, l=0.5893):
         Density, [kg/m³]
     T : float
         Temperature, [K]
-    l : float, optional
+    lr : float, optional
         Light Wavelength, [μm]
 
     Returns
@@ -1011,7 +1014,7 @@ def _Refractive(rho, T, l=0.5893):
         Refractive index, [-]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 0 ≤ ρ ≤ 1060
@@ -1032,14 +1035,16 @@ def _Refractive(rho, T, l=0.5893):
     http://www.iapws.org/relguide/rindex.pdf
     """
     # Check input parameters
-    if rho < 0 or rho > 1060 or T < 261.15 or T > 773.15 or l < 0.2 or l > 1.1:
+    if rho < 0 or rho > 1060 or \
+            T < 261.15 or T > 773.15 or \
+            lr < 0.2 or lr > 1.1:
         raise NotImplementedError("Incoming out of bound")
 
     Lir = 5.432937
     Luv = 0.229202
     d = rho/1000.
     Tr = T/273.15
-    L = l/0.589
+    L = lr/0.589
     a = [0.244257733, 0.974634476e-2, -0.373234996e-2, 0.268678472e-3,
          0.158920570e-2, 0.245934259e-2, 0.900704920, -0.166626219e-1]
     A = d*(a[0]+a[1]*d+a[2]*Tr+a[3]*L**2*Tr+a[4]/L**2+a[5]/(L**2-Luv**2)+a[6]/(
@@ -1063,7 +1068,7 @@ def _Kw(rho, T):
         Ionization constant in -log10(kw), [-]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 0 ≤ ρ ≤ 1250
@@ -1117,7 +1122,7 @@ def _Conductivity(rho, T):
         Electrolytic conductivity, [S/m]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 600 ≤ ρ ≤ 1200
@@ -1134,27 +1139,41 @@ def _Conductivity(rho, T):
     Supercritical Water from 0°C to 800°C and Pressures up to 1000 MPa,
     http://www.iapws.org/relguide/conduct.pdf
     """
-    # FIXME: Dont work
+    # density in g/l
     rho_ = rho/1000
-    kw = 10**-_Kw(rho, T)
+
+    # This guideline predates the current standard on the ionization constant,
+    # therefore the standard accepted at that time must be used in order to
+    # obtain the values of the tables for testing.
+    # Marshall, W.L., Franck, E.U.
+    # Ion product of water substance, 0-1000ºC, 1-10,000 bars New International
+    # Formulation and its background
+    # J. Phys. Chem. Ref. Data 10(2) (1981) 295-304
+    # doi: 10.1063/1.555643
+
+    # Eq 4
+    kw = 10**(-4.098 - 3245.2/T + 2.2362e5/T**2 - 3.984e7/T**3 +
+              (13.957 - 1262.3/T + 8.5641e5/T**2)*log10(rho_))
+
+    # kw = 10**-_Kw(rho, T)
 
     A = [1850., 1410., 2.16417e-6, 1.81609e-7, -1.75297e-9, 7.20708e-12]
     B = [16., 11.6, 3.26e-4, -2.3e-6, 1.1e-8]
     t = T-273.15
 
-    Loo = A[0]-1/(1/A[1]+sum([A[i+2]*t**(i+1) for i in range(4)]))      # Eq 5
-    rho_h = B[0]-1/(1/B[1]+sum([B[i+2]*t**(i+1) for i in range(3)]))    # Eq 6
+    Loo = A[0]-1/(1/A[1] + A[2]*t + A[3]*t**2 + A[4]*t**3 + A[5]*t**4)   # Eq 5
+    rho_h = B[0]-1/(1/B[1] + B[2]*t + B[3]*t**2 + B[4]*t**3)             # Eq 6
 
     # Eq 4
     L_o = (rho_h-rho_)*Loo/rho_h
 
     # Eq 1
-    k = 100*1e-3*L_o*kw**0.5*rho_
-    return k
+    k = 1e-3*L_o*kw**0.5*rho_
+    return k*1e2
 
 
 # Heavy water transport properties
-def _D2O_Viscosity(rho, T):
+def _D2O_Viscosity(rho, T, fase=None, drho=None):
     """Equation for the Viscosity of heavy water
 
     Parameters
@@ -1163,6 +1182,10 @@ def _D2O_Viscosity(rho, T):
         Density, [kg/m³]
     T : float
         Temperature, [K]
+    fase: dict, optional for calculate critical enhancement
+        phase properties
+    drho: float, optional for calculate critical enhancement
+        [∂ρ/∂P]T at reference state,
 
     Returns
     -------
@@ -1178,32 +1201,72 @@ def _D2O_Viscosity(rho, T):
 
     References
     ----------
-    IAPWS, Revised Release on Viscosity and Thermal Conductivity of Heavy
-    Water Substance, http://www.iapws.org/relguide/TransD2O-2007.pdf
+    IAPWS, Release on the IAPWS Formulation 2020 for the Viscosity of Heavy
+    Water, http://iapws.org/relguide/D2Ovisc.pdf
     """
-    Tr = T/643.847
-    rhor = rho/358.0
+    Tr = T/Tc_D2O
+    rhor = rho/356
 
-    no = [1.0, 0.940695, 0.578377, -0.202044]
-    fi0 = Tr**0.5/sum([n/Tr**i for i, n in enumerate(no)])
+    # Eq 11, viscosity in the dilute-gas limit
+    no = 0.889754+61.22217*Tr-44.8866*Tr**2+111.5812*Tr**3+3.547412*Tr**4
+    do = 0.79637+2.38127*Tr-0.33463*Tr**2+2.669*Tr**3+0.000211366*Tr**4
+    mu0 = Tr**0.5 * no/do
 
-    Li = [0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 0, 1, 2, 5, 0, 1, 2, 3, 0, 1, 3,
-          5, 0, 1, 5, 3]
-    Lj = [0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4,
-          4, 5, 5, 5, 6]
-    Lij = [0.4864192, -0.2448372, -0.8702035, 0.8716056, -1.051126,
-           0.3458395, 0.3509007, 1.315436, 1.297752, 1.353448, -0.2847572,
-           -1.037026, -1.287846, -0.02148229, 0.07013759, 0.4660127,
-           0.2292075, -0.4857462, 0.01641220, -0.02884911, 0.1607171,
-           -.009603846, -.01163815, -.008239587, 0.004559914, -0.003886659]
+    # Eq 12
+    hi = [0, 2, 3, 4, 5, 6, 0, 1, 3, 4, 6, 0, 1, 5, 0, 1, 2, 5, 6, 0, 2, 3, 5,
+          2, 2]
+    hj = [0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 2, 2, 2, 3, 3, 3, 3, 3, 4, 4, 4, 4,
+          5, 6]
+    Hij = [0.510953, -0.558947, -2.718820, 0.480990, 2.404510, -1.824320,
+           0.275847, 0.762957, 1.760340, 0.0819086, 1.417750, -0.228148,
+           -0.321497, -2.302500, 0.0661035, 0.0449393, 1.466670, 0.938984,
+           -0.108354, -0.00481265, -1.545710, -0.0570938, -0.0753783, 0.553080,
+           -0.0650201]
 
-    arr = [lij*(1./Tr-1)**i*(rhor-1)**j for i, j, lij in zip(Li, Lj, Lij)]
-    fi1 = exp(rhor*sum(arr))
+    arr = [(1/Tr-1)**i*(rhor-1)**j*hij for i, j, hij in zip(hi, hj, Hij)]
+    mu1 = exp(rhor*sum(arr))
 
-    return 55.2651e-6*fi0*fi1
+    # Critical enhancement
+    if rho and fase and drho:
+        qc = 1/1.9
+        qd = 1/0.4
+
+        # Eq 21
+        DeltaX = Pc_D2O*rhor**2*(fase.drhodP_T/rho-drho/rho*1.5/Tr)
+        if DeltaX < 0:
+            DeltaX = 0
+
+        # Eq 20
+        X = 0.13*(DeltaX/0.06)**(0.63/1.239)
+
+        if X <= 0.03021806692:
+            # Eq 15
+            Y = qc/5*X*(qd*X)**5*(1-qc*X+(qc*X)**2-765/504*(qd*X)**2)
+
+        else:
+            Fid = acos((1+qd**2*X**2)**-0.5)                            # Eq 17
+            w = abs((qc*X-1)/(qc*X+1))**0.5*tan(Fid/2)                  # Eq 19
+
+            # Eq 18
+            if qc*X > 1:
+                Lw = log((1+w)/(1-w))
+            else:
+                Lw = 2*atan(abs(w))
+
+            # Eq 16
+            Y = sin(3*Fid)/12-sin(2*Fid)/4/qc/X+(1-5/4*(qc*X)**2)/(
+                qc*X)**2*sin(Fid)-((1-3/2*(qc*X)**2)*Fid-abs((
+                    qc*X)**2-1)**1.5*Lw)/(qc*X)**3
+
+        # Eq 14
+        mu2 = exp(0.068*Y)
+    else:
+        mu2 = 1
+
+    return mu0*mu1*mu2*1e-6
 
 
-def _D2O_ThCond(rho, T):
+def _D2O_ThCond(rho, T, fase=None, drho=None):
     """Equation for the thermal conductivity of heavy water
 
     Parameters
@@ -1212,6 +1275,10 @@ def _D2O_ThCond(rho, T):
         Density, [kg/m³]
     T : float
         Temperature, [K]
+    fase: dict, optional for calculate critical enhancement
+        phase properties
+    drho: float, optional for calculate critical enhancement
+        [∂ρ/∂P]T at reference state,
 
     Returns
     -------
@@ -1227,29 +1294,59 @@ def _D2O_ThCond(rho, T):
 
     References
     ----------
-    IAPWS, Revised Release on Viscosity and Thermal Conductivity of Heavy
-    Water Substance, http://www.iapws.org/relguide/TransD2O-2007.pdf
+    IAPWS, Release on the IAPWS Formulation 2021 for the Thermal Conductivity
+    of Heavy Water, http://iapws.org/relguide/D2OThCond.pdf
     """
-    rhor = rho/358
+    d = rho/356
     Tr = T/643.847
-    tau = Tr/(abs(Tr-1.1)+1.1)
 
-    no = [1.0, 37.3223, 22.5485, 13.0465, 0.0, -2.60735]
-    Lo = sum([Li*Tr**i for i, Li in enumerate(no)])
+    # Eq 16
+    no = [1, 3.3620798, -1.0191198, 2.8518117]
+    do = [0.10779213, -0.034637234, 0.036603464, 0.0091018912]
+    k0 = Tr**0.5 * sum(Li*Tr**i for i, Li in enumerate(no)) / \
+        sum(Li*Tr**i for i, Li in enumerate(do))
 
-    nr = [483.656, -191.039, 73.0358, -7.57467]
-    Lr = -167.31*(1-exp(-2.506*rhor))+sum(
-        [Li*rhor**(i+1) for i, Li in enumerate(nr)])
+    # Eq 17
+    li = [0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3,
+          3, 4, 4, 4, 4, 4, 4]
+    lj = [0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4,
+          5, 0, 1, 2, 3, 4, 5]
+    nij = [1.50933576, -0.65831078, 0.111174263, 0.140185152, -0.0656227722,
+           0.00785155213, 2.8414715, -2.9826577, 1.34357932, -0.599233641,
+           0.28116337, -0.0533292833, 4.86095723, -6.19784468, 2.20941867,
+           0.224691518, -0.322191265, 0.0596204654, 2.06156007, -3.48612456,
+           1.47962309, 0.625101458, -0.56123225, 0.0974446139, -2.06105687,
+           0.416240028, 2.92524513, -2.81703583, 1.00551476, -0.127884416]
 
-    f1 = exp(0.144847*Tr-5.64493*Tr**2)
-    f2 = exp(-2.8*(rhor-1)**2)-0.080738543*exp(-17.943*(rhor-0.125698)**2)
-    f3 = 1+exp(60*(tau-1)+20)
-    f4 = 1+exp(100*(tau-1)+15)
-    Lc = 35429.6*f1*f2*(1+f2**2*(5e9*f1**4/f3+3.5*f2/f4))
+    k1 = exp(d*sum((1/Tr-1)**i * n*(d-1)**j for i, j, n in zip(li, lj, nij)))
 
-    Ll = -741.112*f1**1.2*(1-exp(-(rhor/2.5)**10))
+    # Critical enhancement
+    if rho and fase and drho:
+        Rg = 0.415151994
+        DeltaX = d*(Pc_D2O/rhoc_D2O*fase.drhodP_T-Pc_D2O/rhoc_D2O*drho*1.5/Tr)
+        if DeltaX < 0:
+            DeltaX = 0
 
-    return 0.742128e-3*(Lo+Lr+Lc+Ll)
+        X = 0.13*(DeltaX/0.06)**(0.63/1.239)                            # Eq 22
+        y = X/0.36                                                      # Eq 20
+
+        # Eq 19
+        if y < 1.2e-7:
+            Z = 0
+        else:
+            Z = 2/pi/y*(((1-1/fase.cp_cv)*atan(y)+y/fase.cp_cv)-(
+                1-exp(-1/(1/y+y**2/3/d**2))))
+
+        # Eq 18
+        k2 = 175.987*d*fase.cp/Rg*Tr/fase.mu*1e-6*Z
+
+    else:
+        # No critical enhancement
+        k2 = 0
+
+    # Eq 15
+    k = k0*k1+k2
+    return 1e-3*k
 
 
 def _D2O_Tension(T):
@@ -1266,7 +1363,7 @@ def _D2O_Tension(T):
         Surface tension, [N/m]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 269.65 ≤ T ≤ 643.847
@@ -1286,8 +1383,8 @@ def _D2O_Tension(T):
     Tr = T/643.847
     if 269.65 <= T < 643.847:
         return 1e-3*(238*(1-Tr)**1.25*(1-0.639*(1-Tr)))
-    else:
-        raise NotImplementedError("Incoming out of bound")
+
+    raise NotImplementedError("Incoming out of bound")
 
 
 def _D2O_Sublimation_Pressure(T):
@@ -1304,7 +1401,7 @@ def _D2O_Sublimation_Pressure(T):
         Pressure at sublimation line, [MPa]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 210 ≤ T ≤ 276.969
@@ -1327,8 +1424,8 @@ def _D2O_Sublimation_Pressure(T):
         for a, t in zip(ai, ti):
             suma += a*(1-Tita**t)
         return exp(suma)*0.00066159
-    else:
-        raise NotImplementedError("Incoming out of bound")
+
+    raise NotImplementedError("Incoming out of bound")
 
 
 def _D2O_Melting_Pressure(T, ice="Ih"):
@@ -1349,7 +1446,7 @@ def _D2O_Melting_Pressure(T, ice="Ih"):
         Pressure at melting line, [MPa]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 254.415 ≤ T ≤ 315
@@ -1457,7 +1554,7 @@ def _Henry(T, gas, liquid="H2O"):
         "CH4(D2O)": (288.16, 517.46)}
 
     # Check input parameters
-    if liquid != "D2O" and liquid != "H2O":
+    if liquid not in ("D2O", "H2O"):
         raise NotImplementedError("Solvent liquid unsupported")
     if gas not in limit:
         raise NotImplementedError("Gas unsupported")
@@ -1467,13 +1564,13 @@ def _Henry(T, gas, liquid="H2O"):
         warnings.warn("Temperature out of data of correlation")
 
     if liquid == "D2O":
-        Tc = 643.847
-        Pc = 21.671
+        Tc_ = Tc_D2O
+        Pc_ = 21.671
     else:
-        Tc = 647.096
-        Pc = 22.064
+        Tc_ = Tc
+        Pc_ = Pc
 
-    Tr = T/Tc
+    Tr = T/Tc_
     tau = 1-Tr
 
     # Eq 4
@@ -1484,7 +1581,7 @@ def _Henry(T, gas, liquid="H2O"):
     else:
         ai = [-7.896657, 24.73308, -27.81128, 9.355913, -9.220083]
         bi = [1, 1.89, 2, 3, 3.6]
-    ps = Pc*exp(1/Tr*sum([a*tau**b for a, b in zip(ai, bi)]))
+    ps = Pc_*exp(1/Tr*sum(a*tau**b for a, b in zip(ai, bi)))
 
     # Select values from Table 2
     par = {
@@ -1582,7 +1679,7 @@ def _Kvalue(T, gas, liquid="H2O"):
         "CH4(D2O)": (288.16, 517.46)}
 
     # Check input parameters
-    if liquid != "D2O" and liquid != "H2O":
+    if liquid not in ("D2O", "H2O"):
         raise NotImplementedError("Solvent liquid unsupported")
     if gas not in limit:
         raise NotImplementedError("Gas unsupported")
@@ -1592,11 +1689,11 @@ def _Kvalue(T, gas, liquid="H2O"):
         warnings.warn("Temperature out of data of correlation")
 
     if liquid == "D2O":
-        Tc = 643.847
+        Tc_ = Tc_D2O
     else:
-        Tc = 647.096
+        Tc_ = Tc
 
-    Tr = T/Tc
+    Tr = T/Tc_
     tau = 1-Tr
 
     # Eq 6
@@ -1609,7 +1706,7 @@ def _Kvalue(T, gas, liquid="H2O"):
         ci = [2.7072, 0.58662, -1.3069, -45.663]
         di = [0.374, 1.45, 2.6, 12.3]
         q = -0.024552
-    f = sum([c*tau**d for c, d in zip(ci, di)])
+    f = sum(c*tau**d for c, d in zip(ci, di))
 
     # Select values from Table 2
     par = {"He": (2267.4082, -2.9616, -3.2604, 7.8819),
