@@ -1,5 +1,8 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+# pylint: disable=invalid-name, too-many-branches, too-many-statements
+# pylint: disable=too-many-arguments
+
 """
 Miscelaneous internal utilities. This module include:
 
@@ -63,6 +66,7 @@ def getphase(Tc, Pc, T, P, x, region):
 
 class _fase(object):
     """Class to implement a null phase"""
+
     v = None
     rho = None
 
@@ -171,23 +175,82 @@ def deriv_H(state, z, x, y, fase):
     if y == "rho":
         y = "v"
 
-    dT = {"P": state.P*1000*fase.alfap,
-          "T": 1,
-          "v": 0,
-          "u": fase.cv,
-          "h": fase.cv+state.P*1000*fase.v*fase.alfap,
-          "s": fase.cv/state.T,
-          "g": state.P*1000*fase.v*fase.alfap-fase.s,
-          "a": -fase.s}
-    dv = {"P": -state.P*1000*fase.betap,
-          "T": 0,
-          "v": 1,
-          "u": state.P*1000*(state.T*fase.alfap-1),
-          "h": state.P*1000*(state.T*fase.alfap-fase.v*fase.betap),
-          "s": state.P*1000*fase.alfap,
-          "g": -state.P*1000*fase.v*fase.betap,
-          "a": -state.P*1000}
-    deriv = (dv[z]*dT[y]-dT[z]*dv[y])/(dv[x]*dT[y]-dT[x]*dv[y])
+    if x == "P":
+        dTdx = state.P*1000*fase.alfap
+        dvdx = -state.P*1000*fase.betap
+    elif x == "T":
+        dTdx = 1
+        dvdx = 0
+    elif x == "v":
+        dTdx = 0
+        dvdx = 1
+    elif x == "u":
+        dTdx = fase.cv
+        dvdx = state.P*1000*(state.T*fase.alfap-1)
+    elif x == "h":
+        dTdx = fase.cv+state.P*1000*fase.v*fase.alfap
+        dvdx = state.P*1000*(state.T*fase.alfap-fase.v*fase.betap)
+    elif x == "s":
+        dTdx = fase.cv/state.T
+        dvdx = state.P*1000*fase.alfap
+    elif x == "g":
+        dTdx = state.P*1000*fase.v*fase.alfap-fase.s
+        dvdx = -state.P*1000*fase.v*fase.betap
+    elif x == "a":
+        dTdx = -fase.s
+        dvdx = -state.P*1000
+
+    if y == "P":
+        dTdy = state.P*1000*fase.alfap
+        dvdy = -state.P*1000*fase.betap
+    elif y == "T":
+        dTdy = 1
+        dvdy = 0
+    elif y == "v":
+        dTdy = 0
+        dvdy = 1
+    elif y == "u":
+        dTdy = fase.cv
+        dvdy = state.P*1000*(state.T*fase.alfap-1)
+    elif y == "h":
+        dTdy = fase.cv+state.P*1000*fase.v*fase.alfap
+        dvdy = state.P*1000*(state.T*fase.alfap-fase.v*fase.betap)
+    elif y == "s":
+        dTdy = fase.cv/state.T
+        dvdy = state.P*1000*fase.alfap
+    elif y == "g":
+        dTdy = state.P*1000*fase.v*fase.alfap-fase.s
+        dvdy = -state.P*1000*fase.v*fase.betap
+    elif y == "a":
+        dTdy = -fase.s
+        dvdy = -state.P*1000
+
+    if z == "P":
+        dTdz = state.P*1000*fase.alfap
+        dvdz = -state.P*1000*fase.betap
+    elif z == "T":
+        dTdz = 1
+        dvdz = 0
+    elif z == "v":
+        dTdz = 0
+        dvdz = 1
+    elif z == "u":
+        dTdz = fase.cv
+        dvdz = state.P*1000*(state.T*fase.alfap-1)
+    elif z == "h":
+        dTdz = fase.cv+state.P*1000*fase.v*fase.alfap
+        dvdz = state.P*1000*(state.T*fase.alfap-fase.v*fase.betap)
+    elif z == "s":
+        dTdz = fase.cv/state.T
+        dvdz = state.P*1000*fase.alfap
+    elif z == "g":
+        dTdz = state.P*1000*fase.v*fase.alfap-fase.s
+        dvdz = -state.P*1000*fase.v*fase.betap
+    elif z == "a":
+        dTdz = -fase.s
+        dvdz = -state.P*1000
+
+    deriv = (dvdz*dTdy-dTdz*dvdy)/(dvdx*dTdy-dTdx*dvdy)
     return mul*deriv
 
 
@@ -241,21 +304,86 @@ def deriv_G(state, z, x, y, fase):
         mul = -1/fase.rho**2
         x = "v"
 
-    dT = {"P": 0,
-          "T": 1,
-          "v": fase.v*fase.alfav,
-          "u": fase.cp-state.P*1000*fase.v*fase.alfav,
-          "h": fase.cp,
-          "s": fase.cp/state.T,
-          "g": -fase.s,
-          "a": -state.P*1000*fase.v*fase.alfav-fase.s}
-    dP = {"P": 1,
-          "T": 0,
-          "v": -fase.v*fase.xkappa,
-          "u": fase.v*(state.P*1000*fase.xkappa-state.T*fase.alfav),
-          "h": fase.v*(1-state.T*fase.alfav),
-          "s": -fase.v*fase.alfav,
-          "g": fase.v,
-          "a": state.P*1000*fase.v*fase.xkappa}
-    deriv = (dP[z]*dT[y]-dT[z]*dP[y])/(dP[x]*dT[y]-dT[x]*dP[y])
+    if x == "P":
+        dPdx = 1.0
+        dTdx = 0.0
+    elif x == "T":
+        dPdx = 0.0
+        dTdx = 1.0
+    elif x == "v":
+        dPdx = -fase.v*fase.xkappa
+        dTdx = fase.v*fase.alfav
+    elif x == "u":
+        dPdx = fase.v*(state.P*1000.0*fase.xkappa-state.T*fase.alfav)
+        dTdx = fase.cp-state.P*1000.0*fase.v*fase.alfav
+    elif x == "h":
+        dPdx = fase.v*(1.0-state.T*fase.alfav)
+        dTdx = fase.cp
+    elif x == "s":
+        dPdx = -fase.v * fase.alfav
+        dTdx = fase.cp/state.T
+    elif x == "g":
+        dPdx = fase.v
+        dTdx = -fase.s
+    elif x == "a":
+        dPdx = state.P*1000.0*fase.v*fase.xkappa
+        dTdx = -state.P * 1000.0 * fase.v * fase.alfav - fase.s
+    else:
+        raise ValueError("x must be one of P, T, v, u, h, s, g, a")
+
+    if y == "P":
+        dPdy = 1.0
+        dTdy = 0.0
+    elif y == "T":
+        dPdy = 0.0
+        dTdy = 1.0
+    elif y == "v":
+        dPdy = -fase.v*fase.xkappa
+        dTdy = fase.v*fase.alfav
+    elif y == "u":
+        dPdy = fase.v*(state.P*1000.0*fase.xkappa-state.T*fase.alfav)
+        dTdy = fase.cp-state.P*1000.0*fase.v*fase.alfav
+    elif y == "h":
+        dPdy = fase.v*(1.0-state.T*fase.alfav)
+        dTdy = fase.cp
+    elif y == "s":
+        dPdy = -fase.v * fase.alfav
+        dTdy = fase.cp/state.T
+    elif y == "g":
+        dPdy = fase.v
+        dTdy = -fase.s
+    elif y == "a":
+        dPdy = state.P*1000.0*fase.v*fase.xkappa
+        dTdy = -state.P * 1000.0 * fase.v * fase.alfav - fase.s
+    else:
+        raise ValueError("y must be one of P, T, v, u, h, s, g, a")
+
+    if z == "P":
+        dPdz = 1.0
+        dTdz = 0.0
+    elif z == "T":
+        dPdz = 0.0
+        dTdz = 1.0
+    elif z == "v":
+        dPdz = -fase.v*fase.xkappa
+        dTdz = fase.v*fase.alfav
+    elif z == "u":
+        dPdz = fase.v*(state.P*1000.0*fase.xkappa-state.T*fase.alfav)
+        dTdz = fase.cp-state.P*1000.0*fase.v*fase.alfav
+    elif z == "h":
+        dPdz = fase.v*(1.0-state.T*fase.alfav)
+        dTdz = fase.cp
+    elif z == "s":
+        dPdz = -fase.v * fase.alfav
+        dTdz = fase.cp/state.T
+    elif z == "g":
+        dPdz = fase.v
+        dTdz = -fase.s
+    elif z == "a":
+        dPdz = state.P*1000.0*fase.v*fase.xkappa
+        dTdz = -state.P * 1000.0 * fase.v * fase.alfav - fase.s
+    else:
+        raise ValueError("z must be one of P, T, v, u, h, s, g, a")
+
+    deriv = (dPdz * dTdy - dTdz * dPdy) / (dPdx * dTdy - dTdx * dPdy)
     return mul*deriv
