@@ -1,5 +1,8 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+# pylint: disable=invalid-name
+# pylint: disable=too-many-locals, too-many-statements
+
 """
 Module with Ammonia-water mixture properties and related properties. The module
 include:
@@ -14,7 +17,7 @@ from __future__ import division
 from math import exp, log, pi
 import warnings
 
-from scipy.constants import Boltzmann as kb
+from scipy.constants import Boltzmann
 from .iapws95 import MEoS, IAPWS95, mainClassDoc
 
 
@@ -30,6 +33,7 @@ class NH3(MEoS):
     R134a, R152a, and R123. Springer-Verlag, Berlin, 1994.
     http://doi.org/10.1007/978-3-642-79400-1
     """
+
     name = "ammonia"
     CASNumber = "7664-41-7"
     formula = "NH3"
@@ -114,7 +118,7 @@ class NH3(MEoS):
 
         # Eq 4
         a = [4.99318220, -0.61122364, 0.0, 0.18535124, -0.11160946]
-        omega = exp(sum([ai*log(T_)**i for i, ai in enumerate(a)]))
+        omega = exp(sum(ai*log(T_)**i for i, ai in enumerate(a)))
 
         # Eq 2, Zero-Density Limit
         muo = 2.1357*(T*self.M)**0.5/sigma**2/omega
@@ -124,7 +128,7 @@ class NH3(MEoS):
               -0.13019164e5, 0.33414230e5, -0.58711743e5, 0.71426686e5,
               -0.59834012e5, 0.33652741e5, -0.1202735e5, 0.24348205e4,
               -0.20807957e3]
-        Bn = 0.6022137*sigma**3*sum([c*T_**(-i/2) for i, c in enumerate(cv)])
+        Bn = 0.6022137*sigma**3*sum(c*T_**(-i/2) for i, c in enumerate(cv))
         # Eq 7
         mub = Bn*muo*rho
 
@@ -133,7 +137,7 @@ class NH3(MEoS):
                1.67668649e-4, -1.49710093e-4, 0.77012274e-4]
         ji = [2, 4, 0, 1, 2, 3, 4]
         ii = [2, 2, 3, 3, 4, 4, 4]
-        mur = sum([d/T_**j*rho**i for d, j, i in zip(dij, ji, ii)])
+        mur = sum(d/T_**j*rho**i for d, j, i in zip(dij, ji, ii))
 
         # Eq 1
         mu = muo + mub + mur
@@ -172,19 +176,19 @@ class NH3(MEoS):
 
         # Eq 6
         no = [0.3589e-1, -0.1750e-3, 0.4551e-6, 0.1685e-9, -0.4828e-12]
-        Lo = sum([n*T**i for i, n in enumerate(no)])
+        Lo = sum(n*T**i for i, n in enumerate(no))
 
         # Eq 7
         nb = [0.16207e-3, 0.12038e-5, -0.23139e-8, 0.32749e-11]
-        L_ = sum([n*rho**(i+1) for i, n in enumerate(nb)])
+        L_ = sum(n*rho**(i+1) for i, n in enumerate(nb))
 
         # Critical enchancement
         t = abs(T-405.4)/405.4
         dPT = 1e5*(2.18-0.12/exp(17.8*t))
         nb = 1e-5*(2.6+1.6*t)
 
-        DL = 1.2*kb*T**2/6/pi/nb/(1.34e-10/t**0.63*(1+t**0.5))*dPT**2 * \
-            0.423e-8/t**1.24*(1+t**0.5/0.7)
+        DL = 1.2*Boltzmann*T**2/6/pi/nb/(1.34e-10/t**0.63*(1+t**0.5)) * \
+            dPT**2 * 0.423e-8/t**1.24*(1+t**0.5/0.7)
 
         # Add correction for entire range of temperature, Eq 10
         DL *= exp(-36*t**2)
@@ -204,6 +208,7 @@ class NH3(MEoS):
 
 
 class H2ONH3(object):
+    """Ammonia-water mixtures."""
 
     # TODO: Add equilibrium routine
 
@@ -243,7 +248,7 @@ class H2ONH3(object):
         Properties of Ammonia-Water Mixtures,
         http://www.iapws.org/relguide/nh3h2o.pdf, Table 4
         """
-        # FIXME: The values are good, bad difer by 1%, a error I can find
+        # FIXME: The values are good, bad difer by 1%, a error I can´t find
         # In Pressure happen and only use fird
 
         M = (1-x)*IAPWS95.M + x*NH3.M
@@ -277,15 +282,16 @@ class H2ONH3(object):
         prop["a"] = prop["u"]-T*prop["s"]
         cvR = -tau0**2*fiott - tau**2*firtt
         prop["cv"] = R*cvR
-        prop["cp"] = R*(cvR+(1+delta*fird-delta*tau*firdt)**2 /
-                        (1+2*delta*fird+delta**2*firdd))
-        prop["w"] = (R*T*1000*(1+2*delta*fird+delta**2*firdd +
-                               (1+delta*fird-delta*tau*firdt)**2 / cvR))**0.5
+        prop["cp"] = R*(cvR+(1+delta*fird-delta*tau*firdt)**2
+                        / (1+2*delta*fird+delta**2*firdd))
+        prop["w"] = (R*T*1000*(1+2*delta*fird+delta**2*firdd
+                               + (1+delta*fird-delta*tau*firdt)**2 / cvR))**0.5
         prop["fugH2O"] = Z*exp(fir+delta*fird-x*F)
         prop["fugNH3"] = Z*exp(fir+delta*fird+(1-x)*F)
         return prop
 
-    def _phi0(self, rho, T, x):
+    @staticmethod
+    def _phi0(rho, T, x):
         """Ideal gas Helmholtz energy of binary mixtures and derivatives
 
         Parameters
@@ -414,22 +420,34 @@ class H2ONH3(object):
         Properties of Ammonia-Water Mixtures,
         http://www.iapws.org/relguide/nh3h2o.pdf, Eq 3
         """
-
         # Temperature reducing value, Eq 4
         Tc12 = 0.9648407/2*(IAPWS95.Tc+NH3.Tc)
         Tn = (1-x)**2*IAPWS95.Tc + x**2*NH3.Tc + 2*x*(1-x**1.125455)*Tc12
+
+        # sympy.diff("(1-x)^2*Tc1 + x^2*Tc2 + 2*x*(1-x^a)*Tc12", "x")
+        # Out[]: Tc1*(2*x - 2) - 2*Tc12*a*x**a + 2*Tc12*(1 - x**a) + 2*Tc2*x
         dTnx = -2*IAPWS95.Tc*(1-x) + 2*x*NH3.Tc + 2*Tc12*(1-x**1.125455) - \
             2*Tc12*1.12455*x**1.12455
 
         # Density reducing value, Eq 5
         b = 0.8978069
-        rhoc12 = 1/(1.2395117/2*(1/IAPWS95.rhoc+1/NH3.rhoc))
-        rhon = 1/((1-x)**2/IAPWS95.rhoc + x**2/NH3.rhoc +
-                  2*x*(1-x**b)/rhoc12)
-        drhonx = -(2*b*x**b/rhoc12 + 2*(1-x**b)/rhoc12 +
-                   2*x/NH3.rhoc - 2*(1-x)/IAPWS95.rhoc)/(
-                       2*x*(1-x**b)/rhoc12 + x**2/NH3.rhoc +
-                       (1-x)**2/IAPWS95.rhoc)**2
+        rhoc1m = IAPWS95.rhoc/(IAPWS95.M/1000)
+        rhoc2m = NH3.rhoc/(NH3.M/1000)
+        M = (1-x)*IAPWS95.M + x*NH3.M
+
+        rhoc12 = 1/(1.2395117/2*(1/rhoc1m + 1/rhoc2m))
+        rhonm = 1/((1-x)**2/rhoc1m + x**2/rhoc2m
+                   + 2*x*(1-x**b)/rhoc12)
+        rhon = rhonm*M/1000
+
+        # sympy.diff("1/((1-x)^2/rhoc1 + x^2/rhoc2 + 2*x*(1-x^b)/rhoc12)", "x")
+        # Out[]: (2*b*x**b/rhoc12 - 2*x/rhoc2 - 2*(1 - x**b)/rhoc12 -
+                # (2*x - 2)/rhoc1)/(x**2/rhoc2 + 2*x*(1 - x**b)/rhoc12 +
+                # (1 - x)**2/rhoc1)**2
+        drhonx = M/1000*(2*b*x**b/rhoc12 - 2*(1-x**b)/rhoc12
+                         - 2*x/rhoc2m + 2*(1-x)/rhoc1m)/(
+                         2*x*(1-x**b)/rhoc12 + x**2/rhoc2m
+                         + (1-x)**2/rhoc1m)**2
 
         tau = Tn/T
         delta = rho/rhon
@@ -456,7 +474,8 @@ class H2ONH3(object):
             tau/Tn*dTnx*prop["firt"]
         return prop
 
-    def _Dphir(self, tau, delta, x):
+    @staticmethod
+    def _Dphir(tau, delta, x):
         """Departure function to the residual contribution to the free
         Helmholtz energy
 
@@ -577,7 +596,7 @@ def Ttr(x):
         Triple point temperature, [K]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 0 ≤ x ≤ 1
@@ -589,13 +608,13 @@ def Ttr(x):
     http://www.iapws.org/relguide/nh3h2o.pdf, Eq 9
     """
     if 0 <= x <= 0.33367:
-        Ttr = 273.16*(1-0.3439823*x-1.3274271*x**2-274.973*x**3)
+        Tr = 273.16*(1-0.3439823*x-1.3274271*x**2-274.973*x**3)
     elif 0.33367 < x <= 0.58396:
-        Ttr = 193.549*(1-4.987368*(x-0.5)**2)
+        Tr = 193.549*(1-4.987368*(x-0.5)**2)
     elif 0.58396 < x <= 0.81473:
-        Ttr = 194.38*(1-4.886151*(x-2/3)**2+10.37298*(x-2/3)**3)
+        Tr = 194.38*(1-4.886151*(x-2/3)**2+10.37298*(x-2/3)**3)
     elif 0.81473 < x <= 1:
-        Ttr = 195.495*(1-0.323998*(1-x)-15.87560*(1-x)**4)
+        Tr = 195.495*(1-0.323998*(1-x)-15.87560*(1-x)**4)
     else:
         raise NotImplementedError("Incoming out of bound")
-    return Ttr
+    return Tr
