@@ -1,5 +1,9 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+# pylint: disable=invalid-name
+# pylint: disable=too-many-lines, too-many-locals, too-many-statements
+# pylint: disable=too-many-instance-attributes, too-many-boolean-expressions
+
 """
 Module with Air-water mixture properties and related properties. The module
 include:
@@ -19,7 +23,7 @@ import warnings
 
 from scipy.optimize import fsolve
 
-from ._iapws import M as Mw
+from ._iapws import M as MW
 from ._iapws import _Ice
 from ._utils import deriv_G
 from .iapws95 import MEoS, IAPWS95, mainClassDoc
@@ -57,7 +61,7 @@ def _virial(T):
             * Cawwtt: d²Caww/dT², [m⁶/molK²]
 
     Notes
-    ------
+    -----
     Raise :class:`Warning` if T isn't in range of validity:
 
         * Baa: 60 ≤ T ≤ 2000
@@ -108,8 +112,6 @@ def _virial(T):
     vir = wt._virial(T)
     Bww = vir["B"]/wt.rhoc*wt.M
     Cwww = vir["C"]/wt.rhoc**2*wt.M**2
-    dBwwt = -vir["Bt"]*wt.Tc/wt.rhoc*wt.M/T**2
-    dCwwwt = -vir["Ct"]*wt.Tc/wt.rhoc**2*wt.M**2/T**2
 
     # Table 3
     ai = [0.482737e-3, 0.105678e-2, -0.656394e-2, 0.294442e-1, -0.319317e-1]
@@ -117,55 +119,49 @@ def _virial(T):
     ci = [66.5687, -238.834, -176.755]
     di = [-0.237, -1.048, -3.183]
 
-    Baw = 1e-6*sum([c*T_**d for c, d in zip(ci, di)])                  # Eq 7
-    Caaw = 1e-6*sum([a/T_**i for i, a in enumerate(ai)])               # Eq 8
-    Caww = -1e-6*exp(sum([b/T_**i for i, b in enumerate(bi)]))         # Eq 9
+    Baw = 1e-6*sum(c*T_**d for c, d in zip(ci, di))                  # Eq 7
+    Caaw = 1e-6*sum(a/T_**i for i, a in enumerate(ai))               # Eq 8
+    Caww = -1e-6*exp(sum(b/T_**i for i, b in enumerate(bi)))         # Eq 9
 
     # Eq T56
-    Bawt = 1e-6*T_/T*sum([c*d*T_**(d-1) for c, d in zip(ci, di)])
+    Bawt = 1e-6*T_/T*sum(c*d*T_**(d-1) for c, d in zip(ci, di))
     # Eq T57
     Bawtt = 1e-6*T_**2/T**2*sum(
-        [c*d*(d-1)*T_**(d-2) for c, d in zip(ci, di)])
+        c*d*(d-1)*T_**(d-2) for c, d in zip(ci, di))
     # Eq T59
-    Caawt = -1e-6*T_/T*sum([i*a*T_**(-i-1) for i, a in enumerate(ai)])
+    Caawt = -1e-6*T_/T*sum(i*a*T_**(-i-1) for i, a in enumerate(ai))
     # Eq T60
     Caawtt = 1e-6*T_**2/T**2*sum(
-        [i*(i+1)*a*T_**(-i-2) for i, a in enumerate(ai)])
+        i*(i+1)*a*T_**(-i-2) for i, a in enumerate(ai))
     # Eq T62
-    Cawwt = 1e-6*T_/T*sum([i*b*T_**(-i-1) for i, b in enumerate(bi)]) * \
-        exp(sum([b/T_**i for i, b in enumerate(bi)]))
+    Cawwt = 1e-6*T_/T*sum(i*b*T_**(-i-1) for i, b in enumerate(bi)) * \
+        exp(sum(b/T_**i for i, b in enumerate(bi)))
     # Eq T63
     Cawwtt = -1e-6*T_**2/T**2*((
-        sum([i*(i+1)*b*T_**(-i-2) for i, b in enumerate(bi)]) +
-        sum([i*b*T_**(-i-1) for i, b in enumerate(bi)])**2) *
-        exp(sum([b/T_**i for i, b in enumerate(bi)])))
+        sum(i*(i+1)*b*T_**(-i-2) for i, b in enumerate(bi))
+        + sum(i*b*T_**(-i-1) for i, b in enumerate(bi))**2)
+        * exp(sum(b/T_**i for i, b in enumerate(bi))))
 
     # Virial coefficient for air, using too the general virial procedure
     air = Air()
     vir = air._virial(T)
     Baa = vir["B"]/air.rhoc*air.M
     Caaa = vir["C"]/air.rhoc**2*air.M**2
-    dBaat = -vir["Bt"]*air.Tc/air.rhoc*air.M/T**2
-    dCaaat = -vir["Ct"]*air.Tc/air.rhoc**2*air.M**2/T**2
 
     prop = {}
-    prop["Baa"] = Baa/1e3
+    prop["Baa"] = Baa/1000
     prop["Baw"] = Baw
-    prop["Bww"] = Bww/1e3
+    prop["Bww"] = Bww/1000
     prop["Caaa"] = Caaa/1e6
     prop["Caaw"] = Caaw
     prop["Caww"] = Caww
     prop["Cwww"] = Cwww/1e6
-    prop["dBawt"] = Bawt
-    prop["dBawtt"] = Bawtt
-    prop["dCaawt"] = Caawt
-    prop["dCaawtt"] = Caawtt
-    prop["dCawwt"] = Cawwt
-    prop["dCawwtt"] = Cawwtt
-    prop["dBaat"] = dBaat/1e3
-    prop["dBwwt"] = dBwwt/1e3
-    prop["dCaaat"] = dCaaat/1e6
-    prop["dCwwwt"] = dCwwwt/1e6
+    prop["Bawt"] = Bawt
+    prop["Bawtt"] = Bawtt
+    prop["Caawt"] = Caawt
+    prop["Caawtt"] = Caawtt
+    prop["Cawwt"] = Cawwt
+    prop["Cawwtt"] = Cawwtt
     return prop
 
 
@@ -187,7 +183,7 @@ def _fugacity(T, P, x):
         fugacity coefficient, [MPa]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in range of validity:
 
         * 193 ≤ T ≤ 473
@@ -208,9 +204,9 @@ def _fugacity(T, P, x):
     """
     # Check input parameters
     if T < 193 or T > 473 or P < 0 or P > 5 or x < 0 or x > 1:
-        raise(NotImplementedError("Input not in range of validity"))
+        raise NotImplementedError("Input not in range of validity")
 
-    R = 8.314462  # J/molK
+    Rg = 8.314462  # J/molK
 
     # Virial coefficients
     vir = _virial(T)
@@ -225,13 +221,18 @@ def _fugacity(T, P, x):
         (x*(3*x-4)*vir["Bww"]+2*(1-x)*(3*x-2)*vir["Baw"]+3*(1-x)**2*vir["Baa"])
 
     # Eq 2
-    fv = x*P*exp(beta*P*1e6/R/T+0.5*gamma*(P*1e6/R/T)**2)
+    fv = x*P*exp(beta*P*1e6/Rg/T+0.5*gamma*(P*1e6/Rg/T)**2)
     return fv
 
 
 class MEoSBlend(MEoS):
-    """Special meos class to implement pseudocomponent blend and defining its
-    ancillary dew and bubble point"""
+    """
+    Special meos class to implement pseudocomponent blend and defining its
+    ancillary dew and bubble point
+    """
+
+    _blend = {}
+
     @classmethod
     def _dewP(cls, T):
         """Using ancillary equation return the pressure of dew point"""
@@ -273,6 +274,7 @@ class Air(MEoSBlend):
     2000 K at Pressures to 2000 MPa. J. Phys. Chem. Ref. Data 29, 331 (2000).
     http://dx.doi.org/10.1063/1.1285884
     """
+
     name = "air"
     CASNumber = "1"
     formula = "N2+Ar+O2"
@@ -294,7 +296,7 @@ class Air(MEoSBlend):
            "titao": [25.36365, 16.90741],
            "ao_exp2": [-0.197938904],
            "titao2": [87.31279],
-           "sum2": [2./3]
+           "sum2": [2./3],
            }
 
     _constants = {
@@ -499,8 +501,8 @@ class Air(MEoSBlend):
 
                 Xq = Xi/qd
                 # Eq 8
-                Omega = 2/pi*((fase.cp-fase.cv)/fase.cp*atan(Xq) +
-                              fase.cv/fase.cp*(Xq))
+                Omega = 2/pi*((fase.cp-fase.cv)/fase.cp*atan(Xq)
+                              + fase.cv/fase.cp*(Xq))
                 # Eq 9
                 Omega0 = 2/pi*(1-exp(-1/(1/Xq+Xq**2/3*rhoc**2/rho**2)))
 
@@ -537,12 +539,14 @@ class HumidAir(object):
         Mass fraction of water in humid air, [kg/kg]
     xw : float
         Mole fraction of water in humid air, [-]
+    HR : float
+        Humidity ratio, Mass fraction of water in dry air, [kg/kg]
 
     Notes
     -----
     * It needs two incoming properties of T, P, rho.
     * v as a alternate input parameter to rho
-    * For composition need one of A, xa, W, xw.
+    * For composition need one of A, xa, W, xw, HR.
 
     The calculated instance has the following properties:
 
@@ -567,13 +571,15 @@ class HumidAir(object):
         * W: Mass fraction of water in humid air, [kg/kg]
         * xa: Mole fraction of dry air, [-]
         * xw: Mole fraction of water, [-]
+        * Pv: Partial pressure of water, [MPa]
         * xa_sat: Mole fraction of dry air at saturation state, [-]
         * mu: Relative chemical potential, [kJ/kg]
         * muw: Chemical potential of water, [kJ/kg]
         * M: Molar mass of humid air, [g/mol]
-        * HR: Humidity ratio, [-]
+        * HR: Humidity ratio, Mass fraction of water in dry air, [kg/kg]
         * RH: Relative humidity, [-]
     """
+
     kwargs = {"T": 0.0,
               "P": 0.0,
               "rho": 0.0,
@@ -581,9 +587,41 @@ class HumidAir(object):
               "A": None,
               "xa": None,
               "W": None,
-              "xw": None}
+              "xw": None,
+              "HR": None}
+
     status = 0
     msg = "Undefined"
+    _mode = None
+    _composition = None
+
+    T = None
+    rho = None
+    v = None
+    P = None
+    s = None
+    cp = None
+    h = None
+    g = None
+    u = None
+    alfav = None
+    betas = None
+    xkappa = None
+    ks= None
+    w = None
+
+    A = None
+    W = None
+    mu = None
+    muw = None
+    M = None
+    HR = None
+    xa = None
+    xw = None
+    Pv = None
+
+    xa_sat = None
+    RH = None
 
     def __init__(self, **kwargs):
         """Constructor, define common constant and initinialice kwargs"""
@@ -593,15 +631,18 @@ class HumidAir(object):
     def __call__(self, **kwargs):
         """Make instance callable to can add input parameter one to one"""
         # Check alernate input parameters
-        if kwargs.get("v", 0):
+        if kwargs.get("v", None) is not None:
             kwargs["rho"] = 1./kwargs["v"]
             del kwargs["v"]
-        if kwargs.get("W", 0):
+        if kwargs.get("W", None) is not None:
             kwargs["A"] = 1-kwargs["W"]
             del kwargs["W"]
-        if kwargs.get("xw", 0):
+        if kwargs.get("xw", None) is not None:
             kwargs["xa"] = 1-kwargs["xw"]
             del kwargs["xw"]
+        if kwargs.get("HR", None) is not None:
+            kwargs["A"] = 1/(1+kwargs["HR"])
+            del kwargs["HR"]
 
         self.kwargs.update(kwargs)
 
@@ -641,7 +682,7 @@ class HumidAir(object):
             A = self.kwargs["A"]
         elif self._composition == "xa":
             xa = self.kwargs["xa"]
-            A = xa/(1-(1-xa)*(1-Mw/Ma))
+            A = xa/(1-(1-xa)*(1-MW/Ma))
 
         # Thermodynamic definition
         if self._mode == "TP":
@@ -689,12 +730,20 @@ class HumidAir(object):
 
         # Saturation related properties
         A_sat = self._eq(self.T, self.P)
-        self.xa_sat = A_sat*Mw/Ma/(1-A_sat*(1-Mw/Ma))
-        self.RH = (1-self.xa)/(1-self.xa_sat)
+        if A_sat:
+            self.xa_sat = A_sat*MW/Ma/(1-A_sat*(1-MW/Ma))
+            self.RH = (1-self.xa)/(1-self.xa_sat)
+        else:
+            self.xa_sat = None
+            self.RH = None
+            self.msg = "Saturation state don't converge"
+            self.status = 3
 
     def derivative(self, z, x, y):
-        """Wrapper derivative for custom derived properties
-        where x, y, z can be: P, T, v, rho, u, h, s, g, a"""
+        """
+        Wrapper derivative for custom derived properties
+        where x, y, z can be: P, T, v, rho, u, h, s, g, a
+        """
         return deriv_G(self, z, x, y, self)
 
     def _eq(self, T, P):
@@ -712,12 +761,17 @@ class HumidAir(object):
         Asat : float
             Saturation mass fraction of dry air in humid air [kg/kg]
         """
+        # ao initial value of air mass fraction for iteration
         if T <= 273.16:
             ice = _Ice(T, P)
             gw = ice["g"]
-        else:
+            ao = 0.99999
+        elif T <= 373.16:
             water = IAPWS95(T=T, P=P)
             gw = water.g
+            ao = 0.99
+        else:
+            raise NotImplementedError("Incoming out of bound")
 
         def f(parr):
             rho, a = parr
@@ -727,11 +781,17 @@ class HumidAir(object):
             muw = fa["fir"]+rho*fa["fird"]-a*fa["fira"]
             return gw-muw, rho**2*fa["fird"]/1000-P
 
-        rinput = fsolve(f, [1, 0.95], full_output=True)
+        air = Air(T=T, P=P)
+        rinput = fsolve(f, [air.rho, ao], full_output=True)
         Asat = rinput[0][1]
-        return Asat
+        if rinput[2] == 1 and 0 <= Asat <= 1:
+            return Asat
 
-    def _prop(self, T, rho, fav):
+        warnings.warn("Convergence failed")
+        print(rinput)
+
+    @staticmethod
+    def _prop(T, rho, fav):
         """Thermodynamic properties of humid air
 
         Parameters
@@ -779,11 +839,12 @@ class HumidAir(object):
         prop["xkappa"] = 1e3/(rho**2*(2*fav["fird"]+rho*fav["firdd"]))  # Eq T8
         prop["ks"] = 1000*fav["firtt"]/rho**2/(                         # Eq T9
             fav["firtt"]*(2*fav["fird"]+rho*fav["firdd"])-rho*fav["firdt"]**2)
-        prop["w"] = (rho**2*1000*(fav["firtt"]*fav["firdd"]-fav["firdt"]**2) /
-                     fav["firtt"]+2*rho*fav["fird"]*1000)**0.5         # Eq T10
+        prop["w"] = (rho**2*1000*(fav["firtt"]*fav["firdd"]-fav["firdt"]**2)
+                     / fav["firtt"]+2*rho*fav["fird"]*1000)**0.5       # Eq T10
         return prop
 
-    def _coligative(self, rho, A, fav):
+    @staticmethod
+    def _coligative(rho, A, fav):
         """Miscelaneous properties of humid air
 
         Parameters
@@ -817,9 +878,9 @@ class HumidAir(object):
         prop = {}
         prop["mu"] = fav["fira"]
         prop["muw"] = fav["fir"]+rho*fav["fird"]-A*fav["fira"]
-        prop["M"] = 1/((1-A)/Mw+A/Ma)
+        prop["M"] = 1/((1-A)/MW+A/Ma)
         prop["HR"] = 1/A-1
-        prop["xa"] = A*Mw/Ma/(1-A*(1-Mw/Ma))
+        prop["xa"] = A*MW/Ma/(1-A*(1-MW/Ma))
         prop["xw"] = 1-prop["xa"]
         return prop
 
@@ -857,7 +918,7 @@ class HumidAir(object):
         Seawater and Ice, Consistent with the IAPWS Formulation 2008 for the
         Thermodynamic Properties of Seawater, Table 6,
         http://www.iapws.org/relguide/SeaAir.html
-        """
+        """  # noqa
         water = IAPWS95()
         rhov = (1-A)*rho
         fv = water._derivDimensional(rhov, T)
@@ -879,8 +940,8 @@ class HumidAir(object):
         # Eq T14
         prop["fird"] = (1-A)**2*fv["fird"]+A**2*fa["fird"]+fmix["fird"]
         # Eq T15
-        prop["firaa"] = rho*(2*fv["fird"]+rhov*fv["firdd"] +
-                             2*fa["fird"]+rhoa*fa["firdd"])+fmix["firaa"]
+        prop["firaa"] = rho*(2*fv["fird"]+rhov*fv["firdd"]
+                             + 2*fa["fird"]+rhoa*fa["firdd"])+fmix["firaa"]
         # Eq T16
         prop["firat"] = -fv["firt"]-rhov*fv["firdt"]+fa["firt"] + \
             rhoa*fa["firdt"]+fmix["firat"]
@@ -895,7 +956,8 @@ class HumidAir(object):
         prop["firdd"] = (1-A)**3*fv["firdd"]+A**3*fa["firdd"]+fmix["firdd"]
         return prop
 
-    def _fmix(self, T, rho, A):
+    @staticmethod
+    def _fmix(T, rho, A):
         r"""Specific Helmholtz energy of air-water interaction
 
         Parameters
@@ -930,45 +992,45 @@ class HumidAir(object):
         Thermodynamic Properties of Seawater, Table 10,
         http://www.iapws.org/relguide/SeaAir.html
         """
-        Ma = Air.M/1000
+        ma = Air.M/1000
         Mw = IAPWS95.M/1000
         vir = _virial(T)
         Baw = vir["Baw"]
-        Bawt = vir["dBawt"]
-        Bawtt = vir["dBawtt"]
+        Bawt = vir["Bawt"]
+        Bawtt = vir["Bawtt"]
         Caaw = vir["Caaw"]
-        Caawt = vir["dCaawt"]
-        Caawtt = vir["dCaawtt"]
+        Caawt = vir["Caawt"]
+        Caawtt = vir["Caawtt"]
         Caww = vir["Caww"]
-        Cawwt = vir["dCawwt"]
-        Cawwtt = vir["dCawwtt"]
+        Cawwt = vir["Cawwt"]
+        Cawwtt = vir["Cawwtt"]
 
         # Eq T45
-        f = 2*A*(1-A)*rho*R*T/Ma/Mw*(Baw+3*rho/4*(A/Ma*Caaw+(1-A)/Mw*Caww))
+        f = 2*A*(1-A)*rho*R*T/ma/Mw*(Baw+3*rho/4*(A/ma*Caaw+(1-A)/Mw*Caww))
         # Eq T46
-        fa = 2*rho*R*T/Ma/Mw*((1-2*A)*Baw+3*rho/4*(
-            A*(2-3*A)/Ma*Caaw+(1-A)*(1-3*A)/Mw*Caww))
+        fa = 2*rho*R*T/ma/Mw*((1-2*A)*Baw+3*rho/4*(
+            A*(2-3*A)/ma*Caaw+(1-A)*(1-3*A)/Mw*Caww))
         # Eq T47
-        ft = 2*A*(1-A)*rho*R/Ma/Mw*(
-            Baw+T*Bawt+3*rho/4*(A/Ma*(Caaw+T*Caawt)+(1-A)/Mw*(Caww+T*Cawwt)))
+        ft = 2*A*(1-A)*rho*R/ma/Mw*(
+            Baw+T*Bawt+3*rho/4*(A/ma*(Caaw+T*Caawt)+(1-A)/Mw*(Caww+T*Cawwt)))
         # Eq T48
-        fd = A*(1-A)*R*T/Ma/Mw*(2*Baw+3*rho*(A/Ma*Caaw+(1-A)/Mw*Caww))
+        fd = A*(1-A)*R*T/ma/Mw*(2*Baw+3*rho*(A/ma*Caaw+(1-A)/Mw*Caww))
         # Eq T49
-        faa = rho*R*T/Ma/Mw*(-4*Baw+3*rho*((1-3*A)/Ma*Caaw-(2-3*A)/Mw*Caww))
+        faa = rho*R*T/ma/Mw*(-4*Baw+3*rho*((1-3*A)/ma*Caaw-(2-3*A)/Mw*Caww))
         # Eq T50
-        fat = 2*rho*R/Ma/Mw*(1-2*A)*(Baw+T*Bawt)+3*rho**2*R/2/Ma/Mw*(
-            A*(2-3*A)/Ma*(Caaw+T*Caawt)+(1-A)*(1-3*A)/Mw*(Caww+T*Cawwt))
+        fat = 2*rho*R/ma/Mw*(1-2*A)*(Baw+T*Bawt)+3*rho**2*R/2/ma/Mw*(
+            A*(2-3*A)/ma*(Caaw+T*Caawt)+(1-A)*(1-3*A)/Mw*(Caww+T*Cawwt))
         # Eq T51
-        fad = 2*R*T/Ma/Mw*((1-2*A)*Baw+3/2*rho*(
-            A*(2-3*A)/Ma*Caaw+(1-A)*(1-3*A)/Mw*Caww))
+        fad = 2*R*T/ma/Mw*((1-2*A)*Baw+3/2*rho*(
+            A*(2-3*A)/ma*Caaw+(1-A)*(1-3*A)/Mw*Caww))
         # Eq T52
-        ftt = 2*A*(1-A)*rho*R/Ma/Mw*(2*Bawt+T*Bawtt+3*rho/4*(
-            A/Ma*(2*Caawt+T*Caawtt)+(1-A)/Mw*(2*Cawwt+T*Cawwtt)))
+        ftt = 2*A*(1-A)*rho*R/ma/Mw*(2*Bawt+T*Bawtt+3*rho/4*(
+            A/ma*(2*Caawt+T*Caawtt)+(1-A)/Mw*(2*Cawwt+T*Cawwtt)))
         # Eq T53
-        ftd = 2*A*(1-A)*R/Ma/Mw*(Baw+T*Bawt+3*rho/2*(
-            A/Ma*(Caaw+T*Caawt)+(1-A)/Mw*(Caww+T*Cawwt)))
+        ftd = 2*A*(1-A)*R/ma/Mw*(Baw+T*Bawt+3*rho/2*(
+            A/ma*(Caaw+T*Caawt)+(1-A)/Mw*(Caww+T*Cawwt)))
         # Eq T54
-        fdd = 3*A*(1-A)*R*T/Ma/Mw*(A/Ma*Caaw+(1-A)/Mw*Caww)
+        fdd = 3*A*(1-A)*R*T/ma/Mw*(A/ma*Caaw+(1-A)/Mw*Caww)
 
         prop = {}
         prop["fir"] = f/1000
