@@ -1,5 +1,9 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+# pylint: disable=invalid-name
+# pylint: disable=too-many-instance-attributes, too-many-boolean-expressions
+# pylint: disable=too-many-locals
+
 """
 IAPWS standard for Seawater IAPWS08 and related functionality. The module
 include:
@@ -25,7 +29,7 @@ import warnings
 from scipy.optimize import fsolve
 
 from .iapws95 import IAPWS95
-from .iapws97 import IAPWS97, _Region1, _Region2
+from .iapws97 import IAPWS97, _Region1, _Region2, _TSat_P
 from ._iapws import _ThCond, Tc, Pc, rhoc, _Ice, _Tension
 from ._utils import deriv_G
 
@@ -123,7 +127,7 @@ class SeaWater(object):
         Haline contraction coefficient, [kg/kg]
 
     Notes
-    ------
+    -----
     :class:`Warning` if input isn't in limit:
 
         * 261 ≤ T ≤ 353
@@ -161,6 +165,7 @@ class SeaWater(object):
     >>> salt.haline
     0.7311487666026304
     """
+
     kwargs = {"T": 0.0,
               "P": 0.0,
               "S": None,
@@ -169,6 +174,31 @@ class SeaWater(object):
     status = 0
     msg = "Undefined"
 
+    T = None
+    P = None
+    rho = None
+    v = None
+    s = None
+    cp = None
+    cv = None
+    h = None
+    u = None
+    a = None
+    alfav = None
+    betas = None
+    xkappa = None
+    ks = None
+    w = None
+    k = None
+
+    sigma = None
+    m = None
+    mu = None
+    muw = None
+    mus = None
+    osm = None
+    haline = None
+
     def __init__(self, **kwargs):
         """Constructor, initinialice kwargs"""
         self.kwargs = SeaWater.kwargs.copy()
@@ -197,7 +227,7 @@ class SeaWater(object):
             pw = self._waterIF97(T, P)
         else:
             pw = self._water(T, P)
-        ps = self._saline(T, P, S)
+        ps = self.saline(T, P, S)
 
         prop = {}
         for key in ps:
@@ -219,8 +249,8 @@ class SeaWater(object):
         self.xkappa = -prop["gpp"]/prop["gp"]
         self.ks = (prop["gtp"]**2-prop["gtt"]*prop["gpp"])/prop["gp"] / \
             prop["gtt"]
-        self.w = prop["gp"]*(prop["gtt"]*1000/(prop["gtp"]**2 -
-                             prop["gtt"]*1000*prop["gpp"]*1e-6))**0.5
+        self.w = prop["gp"]*(prop["gtt"]*1000/(
+            prop["gtp"]**2 - prop["gtt"]*1000*prop["gpp"]*1e-6))**0.5
 
         # Thermal conductivity calculation
         if "thcond" in pw:
@@ -236,7 +266,7 @@ class SeaWater(object):
         try:
             self.sigma = _Tension_SeaWater(T, S)
         except NotImplementedError:
-            self.sigma = None
+            pass
 
         if S:
             self.mu = prop["gs"]
@@ -244,16 +274,12 @@ class SeaWater(object):
             self.mus = prop["g"]+(1-S)*prop["gs"]
             self.osm = -(ps["g"]-S*prop["gs"])/self.m/Rm/T
             self.haline = -prop["gsp"]/prop["gp"]
-        else:
-            self.mu = None
-            self.muw = None
-            self.mus = None
-            self.osm = None
-            self.haline = None
 
     def derivative(self, z, x, y):
-        """Wrapper derivative for custom derived properties
-        where x, y, z can be: P, T, v, u, h, s, g, a"""
+        """
+        Wrapper derivative for custom derived properties
+        where x, y, z can be: P, T, v, u, h, s, g, a
+        """
         return deriv_G(self, z, x, y, self)
 
     @classmethod
@@ -289,8 +315,10 @@ class SeaWater(object):
 
     @classmethod
     def _waterSupp(cls, T, P):
-        """Get properties of pure water using the supplementary release SR7-09,
-        Table4 pag 6"""
+        """
+        Get properties of pure water using the supplementary release SR7-09,
+        Table4 pag 6
+        """
         tau = (T-273.15)/40
         pi = (P-0.101325)/100
 
@@ -339,27 +367,25 @@ class SeaWater(object):
         return prop
 
     @classmethod
-    def _saline(cls, T, P, S):
+    def saline(cls, T, P, S):
         """Eq 4"""
-
         # Check input in range of validity
         if T <= 261 or T > 353 or P <= 0 or P > 100 or S < 0 or S > 0.12:
             warnings.warn("Incoming out of bound")
 
-        S_ = 0.03516504*40/35
         X = (S/S_)**0.5
         tau = (T-273.15)/40
         pi = (P-0.101325)/100
 
-        I = [1, 2, 3, 4, 5, 6, 7, 1, 2, 3, 4, 5, 6, 2, 3, 4, 2, 3, 4, 2, 3, 4,
-             2, 4, 2, 2, 3, 4, 5, 2, 3, 4, 2, 3, 2, 3, 2, 3, 2, 3, 4, 2, 3, 2,
-             3, 2, 2, 2, 3, 4, 2, 3, 2, 3, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2]
-        J = [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4,
-             5, 5, 6, 0, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 4, 4, 0, 0, 0, 1, 1, 2,
-             2, 3, 4, 0, 0, 0, 1, 1, 2, 2, 3, 4, 0, 0, 1, 2, 3, 0, 1, 2]
-        K = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-             0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2,
-             2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 4, 5, 5, 5]
+        Li = [1, 2, 3, 4, 5, 6, 7, 1, 2, 3, 4, 5, 6, 2, 3, 4, 2, 3, 4, 2, 3, 4,
+              2, 4, 2, 2, 3, 4, 5, 2, 3, 4, 2, 3, 2, 3, 2, 3, 2, 3, 4, 2, 3, 2,
+              3, 2, 2, 2, 3, 4, 2, 3, 2, 3, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2]
+        Lj = [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4,
+              5, 5, 6, 0, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 4, 4, 0, 0, 0, 1, 1, 2,
+              2, 3, 4, 0, 0, 0, 1, 1, 2, 2, 3, 4, 0, 0, 1, 2, 3, 0, 1, 2]
+        Lk = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+              0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2,
+              2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 4, 5, 5, 5]
         G = [0.581281456626732e4, 0.141627648484197e4, -0.243214662381794e4,
              0.202580115603697e4, -0.109166841042967e4, 0.374601237877840e3,
              -0.485891069025409e2, 0.851226734946706e3, 0.168072408311545e3,
@@ -387,7 +413,7 @@ class SeaWater(object):
 
         # Calculate only for some salinity
         if S != 0:
-            for i, j, k, gi in zip(I, J, K, G):
+            for i, j, k, gi in zip(Li, Lj, Lk, G):
                 if i == 1:
                     g += gi*X**2*log(X)*tau**j*pi**k
                     gs += gi*(2*log(X)+1)*tau**j*pi**k
@@ -448,11 +474,17 @@ def _Tb(P, S):
         pv = _Region2(T, P)
         gv = pv["h"]-T*pv["s"]
 
-        ps = SeaWater._saline(T, P, S)
+        ps = SeaWater.saline(T, P, S)
         return -ps["g"]+S*ps["gs"]-gw+gv
 
-    Tb = fsolve(f, 300)[0]
-    return Tb
+    try:
+        to = _TSat_P(P)
+    except NotImplementedError:
+        to = 300
+    rinput = fsolve(f, to, full_output=True)
+    Tb = fsolve(f, to)[0]
+    if rinput[2] == 1:
+        return Tb
 
 
 def _Tf(P, S):
@@ -482,11 +514,17 @@ def _Tf(P, S):
 
         gih = _Ice(T, P)["g"]
 
-        ps = SeaWater._saline(T, P, S)
+        ps = SeaWater.saline(T, P, S)
         return -ps["g"]+S*ps["gs"]-gw+gih
 
-    Tf = fsolve(f, 300)[0]
-    return Tf
+    try:
+        to = _TSat_P(P)
+    except NotImplementedError:
+        to = 300
+    rinput = fsolve(f, to, full_output=True)
+    Tf = fsolve(f, to)[0]
+    if rinput[2] == 1:
+        return Tf
 
 
 def _Triple(S):
@@ -520,7 +558,7 @@ def _Triple(S):
         gv = pv["h"]-T*pv["s"]
 
         gih = _Ice(T, P)["g"]
-        ps = SeaWater._saline(T, P, S)
+        ps = SeaWater.saline(T, P, S)
 
         return -ps["g"]+S*ps["gs"]-gw+gih, -ps["g"]+S*ps["gs"]-gw+gv
 
@@ -560,7 +598,7 @@ def _OsmoticPressure(T, P, S):
     def f(Posm):
         pw2 = _Region1(T, P+Posm)
         gw2 = pw2["h"]-T*pw2["s"]
-        ps = SeaWater._saline(T, P+Posm, S)
+        ps = SeaWater.saline(T, P+Posm, S)
         return -ps["g"]+S*ps["gs"]-gw+gw2
 
     Posm = fsolve(f, 0)[0]
@@ -585,7 +623,7 @@ def _ThCond_SeaWater(T, P, S):
         Thermal conductivity excess relative to that of the pure water, [W/mK]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 273.15 ≤ T ≤ 523.15
@@ -621,6 +659,7 @@ def _ThCond_SeaWater(T, P, S):
     DL = a*(1000*S)**(1+b)
     return DL
 
+
 def _Tension_SeaWater(T, S):
     """Equation for the surface tension of seawater
 
@@ -637,7 +676,7 @@ def _Tension_SeaWater(T, S):
         Surface tension, [N/m]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 0 ≤ S ≤ 0.131  for 274.15 ≤ T ≤ 365.15
@@ -653,7 +692,6 @@ def _Tension_SeaWater(T, S):
     IAPWS, Guideline on the Surface Tension of Seawater,
     http://www.iapws.org/relguide/Seawater-Surf.html
     """
-
     # Check input parameters
     if 248.15 < T < 274.15:
         if S < 0 or S > 0.038:
@@ -662,12 +700,13 @@ def _Tension_SeaWater(T, S):
         if S < 0 or S > 0.131:
             raise NotImplementedError("Incoming out of bound")
     else:
-            raise NotImplementedError("Incoming out of bound")
+        raise NotImplementedError("Incoming out of bound")
 
     sw = _Tension(T)
     sigma = sw*(1+3.766e-1*S+2.347e-3*S*(T-273.15))
     return sigma
 
+
 def _solNa2SO4(T, mH2SO4, mNaCl):
     """Equation for the solubility of sodium sulfate in aqueous mixtures of
     sodium chloride and sulfuric acid
@@ -687,7 +726,7 @@ def _solNa2SO4(T, mH2SO4, mNaCl):
         Molal solutility of sodium sulfate, [mol/kg(water)]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 523.15 ≤ T ≤ 623.15
@@ -743,7 +782,7 @@ def _critNaCl(x):
             * rhoc: critical density, [kg/m³]
 
     Notes
-    ------
+    -----
     Raise :class:`NotImplementedError` if input isn't in limit:
 
         * 0 ≤ x ≤ 0.12
@@ -763,8 +802,8 @@ def _critNaCl(x):
         raise NotImplementedError("Incoming out of bound")
 
     T1 = Tc*(1 + 2.3e1*x - 3.3e2*x**1.5 - 1.8e3*x**2)
-    T2 = Tc*(1 + 1.757e1*x - 3.026e2*x**1.5 + 2.838e3*x**2 - 1.349e4*x**2.5 +
-             3.278e4*x**3 - 3.674e4*x**3.5 + 1.437e4*x**4)
+    T2 = Tc*(1 + 1.757e1*x - 3.026e2*x**1.5 + 2.838e3*x**2 - 1.349e4*x**2.5
+             + 3.278e4*x**3 - 3.674e4*x**3.5 + 1.437e4*x**4)
     f1 = (abs(10000*x-10-1)-abs(10000*x-10+1))/4+0.5
     f2 = (abs(10000*x-10+1)-abs(10000*x-10-1))/4+0.5
 
@@ -772,9 +811,9 @@ def _critNaCl(x):
     tc = f1*T1+f2*T2
 
     # Eq 7
-    rc = rhoc*(1 + 1.7607e2*x - 2.9693e3*x**1.5 + 2.4886e4*x**2 -
-               1.1377e5*x**2.5 + 2.8847e5*x**3 - 3.8195e5*x**3.5 +
-               2.0633e5*x**4)
+    rc = rhoc*(1 + 1.7607e2*x - 2.9693e3*x**1.5 + 2.4886e4*x**2
+               - 1.1377e5*x**2.5 + 2.8847e5*x**3 - 3.8195e5*x**3.5
+               + 2.0633e5*x**4)
 
     # Eq 8
     DT = tc-Tc
